"""
This file is part of Visuse.

Copyright (C) 2010 Jose Luis Lopez Pino jllopezpino@gmail.com

Visuse is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Visuse is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Visuse.  If not, see <http://www.gnu.org/licenses/>.

"""
# -*- coding: utf-8 -*-
#!/usr/bin/python

import gdata.youtube
import gdata.youtube.service
import json
from classes import Result, ImageResult, SearchModule
#from config import youtube_key, youtube_order
import config
from difflib import SequenceMatcher

class Youtube(Result, ImageResult):
	def __init__(self, name, url, thumbnail, summary, tags, view_count, user_rating, player_url):
		ImageResult.__init__(self, name, url, thumbnail)
		self.summary = summary
		self.tags = tags
		self.view_count = int(view_count)
		self.user_rating = float(user_rating)
		self.player_url = player_url
		
	def estimateRating(self, search_terms):
		name_point = SequenceMatcher(None, self.name, search_terms).real_quick_ratio()
		views_point = min(1, self.view_count / 20000.0)
		rating_point = self.user_rating / 5
		estimated_rating = name_point + 0.2 * views_point + 0.2 * rating_point
		self.rating = max(self.rating, estimated_rating);
		self.rating = min(1, self.rating);
		#~ print "Name: ", self.name, "\tVisitas: ", self.view_count, "\tRating: ", self.user_rating
		#~ print "Name point: ", name_point, "\tViews Point: ", views_point, "\tRating point:", rating_point
		#~ print "Estimated rating: ", estimated_rating, "\tFinal rating: ", self.rating
		
	
class SearchYoutube(SearchModule):
	def __init__(self):
		SearchModule.__init__(self)

	
	def search(self, search_terms):
		yt_service = gdata.youtube.service.YouTubeService()
		yt_service.developer_key = config.youtube_key
		
		query = gdata.youtube.service.YouTubeVideoQuery()
		query.vq = search_terms
		query.orderby = config.youtube_order
		query.racy = 'include'
		query.max_results = 40
		feed = yt_service.YouTubeQuery(query)
		
		for entry in feed.entry:
			self.add(entry)
			
		for n in range(0, len(self.resultList)):
			self.resultList[n].estimateRating(search_terms)
			
		return self.resultList
	

	def add(self, entry):
		
		#~ thumbnailList = []
		#~ for thumb in entry.media.thumbnail:
			#~ thumbnail = ImageLink(thumb.url, thumb.width, thumb.height)
			#~ thumbnailList.append(thumbnail)
		try:
			media = entry.rating.average
		except AttributeError:
			media = 0
		
		if entry.statistics:
			view_count = entry.statistics.view_count
		else:
			view_count = 0;
		
		self.resultList.append(Youtube( \
			entry.media.title.text, \
			entry.media.player.url, \
			entry.media.thumbnail[-1].url, \
			entry.media.description.text, \
			entry.media.keywords.text, \
			view_count, \
			media, \
			entry.GetSwfUrl() \
		))


#~ sy = SearchYoutube()
#~ sy.search("big bang theory")
#~ print(sy)


